<?php
declare( strict_types=1 );

namespace Ilabs\Eclear_Spot\Remote;

use Exception;
use Ilabs\Eclear_Spot\Model\Spot_File;
use Ilabs\Eclear_Spot\Remote\Dto\Spot_Response_DTO;
use Isolated\Eclear_Spot\Isolated_Guzzlehttp\GuzzleHttp\Exception\GuzzleException;

class Spot_Api_Client {

	const UPLOAD_URL = 'https://dev.eclear-solutions.com/ec-media-be/v1/media/upload';

	const UPLOAD_PROD_URL = 'https://eclear-solutions.com/ec-media-be/v1/media/upload';

	const TEST_URL = 'https://dev.eclear-solutions.com/ec-media-be/v1/media/connection/test';

	const TEST_PROD_URL = 'https://eclear-solutions.com/ec-media-be/v1/media/connection/test';

	/**
	 * @throws GuzzleException
	 * @throws Exception
	 */
	public function upload_file( Spot_File $file ): Spot_Response_DTO {

		$client = eclear_spot()->get_guzzlehttp_client();

		$res = $client->request( 'POST', self::UPLOAD_PROD_URL, [
			'headers'   => [
				'Access_key_id'     => $this->get_acces_key(),
				'Access_key_secret' => $this->get_secret_key(),
			],
			'multipart' => [
				[
					'name'     => 'file',
					'contents' => file_get_contents( $file->get_full_path() ),
					'filename' => basename( $file->get_full_path() ),
				],
			],
		] );

		if ( 200 !== $res->getStatusCode() ) {
			throw new Exception( 'Response code: ' . $res->getStatusCode() );
		}

		return new Spot_Response_DTO( $res->getBody() . ' | ' . basename( $file->get_full_path() ) );
	}

	public function get_acces_key(): string {
		return get_option( 'eclear_spot_main_keys_access_key' );
	}

	public function get_secret_key(): string {
		return get_option( 'eclear_spot_main_keys_secret_key' );
	}

	/**
	 * @throws GuzzleException
	 * @throws Exception
	 */
	public function test( $acces_key = null, $secret_key = null ) {

		$client = eclear_spot()->get_guzzlehttp_client();

		try {
			$res = $client->request( 'GET', self::TEST_PROD_URL, [
				'headers' => [
					'Access_key_id'     => isset( $acces_key ) ? $acces_key : $this->get_acces_key(),
					'Access_key_secret' => isset( $secret_key ) ? $secret_key : $this->get_secret_key(),
				],
			] );
		} catch ( Exception $e ) {

			$res = $client->request( 'GET', self::TEST_URL, [
				'headers' => [
					'Access_key_id'     => isset( $acces_key ) ? $acces_key : $this->get_acces_key(),
					'Access_key_secret' => isset( $secret_key ) ? $secret_key : $this->get_secret_key(),
				],
			] );

		}


		if ( 200 !== $res->getStatusCode() ) {
			throw new Exception( 'Response code: ' . $res->getStatusCode() );

		}


		return $res->getBody();
	}
}
