<?php

namespace Ilabs\Eclear_Spot\Remote\Dto;

use Isolated\Eclear_Spot\Ilabs\Ilabs_Plugin\Common\Wc_Helpers;

class Payload_DTO {

	const EXCHANGERATE = 100000;
	/**
	 * @var Info_DTO
	 */
	private $info;
	/**
	 * @var Data_DTO[]
	 */
	private $data;

	/**
	 * @param int[]
	 *
	 */
	public function build( array $order_ids ): void {

		$data_dto_arr = [];

		foreach ( $order_ids as $order_id ) {
			$wc_order        = wc_get_order( $order_id );
			$products_DTO    = Wc_Helpers::get_products_by_order_id( $order_id );
			$product_dto_arr = [];

			foreach ( $products_DTO as $product_data ) {

                // if product is already deleted - it cause to break loop
				$wc_product  = wc_get_product( $product_data['product_id'] );
				// so we check
				if( $wc_product ) {
					$product_dto = new Product_DTO();
					$product_dto->set_quantity( (int) $product_data['product_quantity'] );
					$product_dto->set_category_of_product( ( function () use ( $wc_product ) {
						$term = get_term( $wc_product->get_category_ids()[0], 'product_cat' );

						return $term->name;
					} )() );//todo nie ma jak uniwersalnie mapować
					$product_dto->set_description( $wc_product->get_description() );
					$product_dto->set_purchasing_price( $wc_product->get_price( false ) * 100 );//todo
					$product_dto->set_retail_price( $wc_product->get_price( false ) * 100 );
					$product_dto->set_sku( $wc_product->get_sku() );
					$product_dto->set_ledger( '' );//not mandatory
					$product_dto->set_taric( '' );//not mandatory
					$product_dto->set_taxrate( $wc_order->get_shipping_tax( false ) * 10 );
					$product_dto_arr[] = $product_dto;
				}
			}

			$data_dto = new Data_DTO();

			$data_dto->set_shipping( '' );
			$data_dto->set_currency( $wc_order->get_currency( 'false' ) );
			$data_dto->set_array_items( $product_dto_arr );
			$data_dto->set_consumer_vat_id( '' );//not mandatory
			$data_dto->set_cust_no( '' );//not mandatory
			$data_dto->set_delivery_address_country( $wc_order->get_shipping_country( false ) );
			$data_dto->set_delivery_address_zip( $wc_order->get_shipping_postcode( false ) );
			$data_dto->set_doc_category( '' );//todo
			$data_dto->set_doc_id( '' );//todo
			$data_dto->set_doc_no( $wc_order->get_id() );
			$data_dto->set_exchange_rate( self::EXCHANGERATE );
			$data_dto->set_ext_order_id( $wc_order->get_id() );
			$data_dto->set_invoice_address_country( $wc_order->get_billing_country() );
			$data_dto->set_invoice_address_zip( $wc_order->get_billing_postcode() );
			$data_dto->set_invoice_amount( 100 * (int) $wc_order->get_total( false ) );
			$data_dto->set_invoice_date( $wc_order->get_date_created( false ) );
			$data_dto->set_platform( '' ); //not mandatory
			$data_dto->set_shipped_from_country( WC()->countries->get_base_country() );
			$data_dto->set_shipping_gdate( '' ); //not mandatory
			$data_dto->set_type( '' ); //not mandatory

			$data_dto_arr[] = $data_dto;
		}

		$info_DTO = new Info_DTO();
		$info_DTO->set_customer_guid( $wc_order->get_customer_id( false ) );
		$info_DTO->set_sub_version( '' ); //not mandatory
		$info_DTO->set_version( '' ); //not mandatory
        


		$this->data = $data_dto_arr;
		$this->info = $info_DTO;
	}


	public function to_array(): array {
		return [
			'info' => $this->info->to_array(),
			'data' => ( function () {
				$items = [];
				foreach ( $this->data as $item ) {
					$items[] = $item->to_array();
				}

				return $items;
			} )(),
		];
	}

	/**
	 * @return Info_DTO
	 */
	public function get_info(): Info_DTO {
		return $this->info;
	}

	/**
	 * @param Info_DTO $info
	 */
	public function set_info( Info_DTO $info ): void {
		$this->info = $info;
	}

	/**
	 * @return Data_DTO[]
	 */
	public function get_data(): array {
		return $this->data;
	}

	/**
	 * @param Data_DTO[] $data
	 */
	public function set_data( array $data ): void {
		$this->data = $data;
	}
}
